'  Description: Sign Restrictions Program in Eviews
'  Programmer: David Stephan, World Bank Macroeconomics and Fiscal Global Practice
'  Last Update: 10/30/2015
'___________________________________________________________________________________________________________________________________________________________________________
tic 'start program timer

'rndseed(12345) 'uncomment for reproducibility

close @all
logmode l

'change path to Eviews sample programs filepath
%path = @runpath
cd %path

include .\routines\VARir.prg 'Impulse Response Code
include .\routines\VARHDecomp.prg 'Historical Decomposition Code
include .\routines\VARFevd.prg 'Variance Decomp Code
include .\routines\VARDraw.prg 'Code to Draw from the posterior distribution of a VAR model
include .\routines\wish_rnd.prg 'Generates a random wishart matrix
include .\routines\mvnrnd.prg 'Draw n random d-dimensional vectors from a multivariate Gaussian distribution
include .\routines\qr.prg 'QR matrix decomposition
include .\routines\VARprocs.prg 'Collects and creates useful objects for VAR analysis
include .\routines\signrestrict.prg 'Runs the sign restriction procedure
include .\routines\medtarget.prg 'Calculates the median target for impulses
include .\routines\ChartArrange.prg 'Rearranges results into nicer WF objects

' create workfile
'============================================
'Open workfile or create new workfile and read in data in this section
wfopen .\data.wf1

'Set Parameters
'============================================
!nsteps = 20 'number of periods impulses calculated for
!nlag=1 'number of lags in the VAR
!ndraws=500 'number of successful draws required
%varname = "VRP" 'name of VAR
!pctilel=0.16 'lower percentile for impulse responses
!pctileu=0.84 'upper percentile for impulse responses
!aux=1 '=1 if you want FEVD and Hdecomp to be computed during the sign restriction procedures
!medt=1 '=1 if you want to calculate the median target
!medaux=1 '=1 if you want to calculate the FEVD and HD using the median target 

'Set exogenous
'============================================
'0 no constant 
'1 constant
'2 constant and trend
'3 constant and trend and trend^2
!setexog=1

if !setexog=0 then
	string g_exog=""
endif 
if !setexog=1 then
	string g_exog="C"
	series const=1
endif
if !setexog=2 then
	string g_exog="C trend"
	series const=1
	series trend=@trend
endif
if !setexog=3 then
	string g_exog="C trend trend2"
	series trend2=@trend^2
endif
if !setexog=4 then 'use if you want to include dummy variables or different specification
	string g_exog="C dum1"
	series const=1
	series dum1=0
                smpl 2008Q1 2010Q4
                dum1=1
endif

' estimate unrestricted VAR 'Change the variables after !nlag to the ones you want in your model
'===========================================================================
'Don't replace the @ {g_exog} terms
var {%varname}.ls 1 !nlag logvrp_var chgdp chcpi fed_rate loan_spread ch_ci_loan @ {g_exog}
!nvar={%varname}.@neqn 'number of variables in VAR system

' set which impulses you want to be accumulated or not (=1 for accumulation)
'==============================================================
vector(!nvar) cumul = 0 
'cumul.fill 1,0,1,1,0,0 (eg of how to set the first, third and 4th impulse to be accum and the rest not)

' Collect and create useful objects for VAR analysis
'==============================================================
call VARprocs

' Set the shocks for the VAR Model
'==============================================================

	!nshocks=2 'How many shocks to you want to define
	
	'Create restriction matrices
	for !r=1 to !nshocks
		matrix(!nvar,3) R{!r}
	next

	'Fill the shock matrices according to the following method 
	'  The first column defines the first period from which the restriction is imposed 
	'  The second column defines the last period to which the restriction is imposed 
	'  The last column defines the sign of the restriction: positive (1) or negative (-1)
	'  To leave unrestricted set all three columns to zero


	string s1="Uncertainty Shock" 'name shock for graphs (expansionary)
	R1.fill(b=r) 1,1,1,1,1,-1,0,0,0,0,0,0,0,0,0,1,1,-1
	
	'R1 = [ 1     1     1    % VRP
 	'       1     1    -1    % GDP
  	'       0     0     0    % CPI
 	'       0     0     0    % Federal Funds Rate
  	'       0     0     0    % Loan Spread
 	'       1     1    -1]   % Loan Amount


	string s2="Monetary Policy Shock" 'name shock for graphs (contractionary)
	R2.fill(b=r) 0,0,0,1,1,-1,1,1,-1,1,1,1,0,0,0,0,0,0
	
	'R2 = [ 0     0     0    % VRP
 	'       1     1    -1    % GDP
  	'       1     1    -1    % CPI
 	'       1     1     1    % Federal Funds Rate
  	'       0     0     0    % Loan Spread
 	'       0     0     0]   % Loan Amount


' Run the sign restriction procedures
'==============================================================
call SignRestrict

' Calc Median Target if wanted
'==============================================================
if !medt=1 then
	call medtarget
endif

' Rearrange Results for Easier Viewing
'==============================================================
call ChartArrange

'Cleanup workfile
'============================================
delete resid* fcomp* eps* csx ctx fs* ft* temp* sigma* invairf* checkall* *_hold temp* exvars exogvars vardata yvars cumul g_exog inv_sigma* matu matuu mconst orthnorm s_exvars sign sresiduals vresiduals ymat varsmpl varmod subr invamult* upperb lowerb s? ser? imps
if !aux=1 then
	delete cumulfevd fevd? hshock? FECD* MSE* PSI* mult* icomp inva_big matnan column 
endif
if !setexog<>0 then 
	delete exmat const 
endif	

'Display length of time taken for the program to run (in seconds).
toc
!elapsed = @toc
logmsg Time Taken: !elapsed seconds


