subroutine SignRestrict
		
	!nzr = 0 'number of zero restrictions
	
	'Determine number of zero restrictions
	for !ss=1 to !nshocks
		matrix(!nvar*3,1) tempsum
		tempsum=@vec(@transpose(R{!ss}))
		tempsum(1,1)=R{!ss}(1,1)
		!countzero=tempsum(1,1)
	
		for !ii=2 to @rows(tempsum)
			!countzero=!countzero + tempsum(!ii,1)
		next
	
		if !countzero = 0 then
			!nzr=!nzr+1
		endif
	next
	
	'Number of non-zero restrictions
	!nsr = !nshocks - !nzr
	
	'Create Matrices to Hold Correct Sign Restriction Impulses
	for !i = 1 to !nshocks
		for !j = 1 to !nvar
			matrix(!ndraws,!nsteps) irf{!i}_{!j}    'the !j response to a !i shock
			matrix(1,!nsteps) mIRF{!i}_{!j}    'the median !j response to a !i shock
			matrix(1,!nsteps) mIRFLow{!i}_{!j}    'the lower percentile !j response to a !i shock
			matrix(1,!nsteps) mIRFUpp{!i}_{!j}    'the upper percentile !j response to a !i shock
			if !aux=1 then
				'FEVD
				matrix(!ndraws,!nsteps) fevd{!j}_{!i}    'FEVD !ith variable due to !jth shock
				matrix(1,!nsteps) mFEVD{!j}_{!i}    'FEVD !ith variable due to !jth shock - median
				matrix(1,!nsteps) mFEVDLow{!j}_{!i}    'FEVD !ith variable due to !jth shock - lower percentile
				matrix(1,!nsteps) mFEVDUpp{!j}_{!i}    'FEVD !ith variable due to !jth shock - upper percentile
				'HDECOMP
				matrix(!ndraws,!nobs+!nlag) hd{!j}_{!i}    'HD !j response to a !i shock
				matrix(1,!nobs+!nlag) mHD{!j}_{!i}    'HD !j response to a !i shock
				matrix(1,!nobs+!nlag) mHDlow{!j}_{!i}    'HD !j response to a !i shock
				matrix(1,!nobs+!nlag) mHDupp{!j}_{!i}    'HD !j response to a !i shock
			endif
		next
	next
	
	''Sign restriction routine
	'==========================================================
	!jj = 0 'accepted draws
	!kkk=0 'counter to stop procedure if restrictions are not working
	!ww = 1 ' index for printing on screen; =1 is intervals of 10 loops shown when completed
	
	while !jj < !ndraws 'do until ndraws succesfully found
	!kkk = !kkk+1
	
		if !jj=0 and !kkk>400*!ndraws then
			logmsg Error: Max number of iterations reached. Try different sign restrictions
			stop	
		endif
	
		'1. Draw a random orthonormal matrix
		if !nzr=0 then 'no zero restrictions
			matrix orthnorm = @mnrnd(!nvar,!nvar)
			call qr(orthnorm)
	
		else 'with zero restrictions
			matrix Q = @identity(!nvar)
			matrix auxQ = @mnrnd(!nsr,!nsr)
			call qr(auxQ)
			matplace(Q,S,!nvar-!nsr+1,!nvar-!nsr+1)
			delete auxQ
		endif
	
		'2. Draw F and sigma from the posterior and set up VAR_draw
		call VARDraw
		matrix invAirf_draw = @cholesky(sigma_draw)*@transpose(S)
	
		'3 Compute IRFs given new rotation
		call VARir(Fcomp_draw,invAirf_draw,cumul)
		
		' ... and check whether sign restrictions are satisfied
		matrix(!nvar,!nshocks) checkall=0
		matrix(!nvar,!nshocks) checkall_flip=0
	
		for !ss=1 to !nshocks
			for !ii = 1 to !nvar
				if R{!ss}(!ii,1) = 0 then 'if no restriction set result =1 (makes easier later calculation)
					checkall(!ii,!ss) = 1
					checkall_flip(!ii,!ss) = 1
				else
	
				if R{!ss}(!ii,1) <> 0 then		
					if R{!ss}(!ii,3) = 1 then
						!check=0
						for !rr=R{!ss}(!ii,1) to R{!ss}(!ii,2)
							if {%irfname}{!ss}(!ii,!rr)>0 then
								!check=!check+1
							endif
						next
						
						if !check=( R{!ss}(!ii,2)-R{!ss}(!ii,1) +1 ) then
							checkall(!ii,!ss) = 1
						endif 
	
						' Check flipped signs
						!check_flip=0
						for !rr=R{!ss}(!ii,1) to R{!ss}(!ii,2)
							if {%irfname}{!ss}(!ii,!rr)<0 then
								!check_flip=!check_flip+1
							endif
						next
	
						if !check_flip=( R{!ss}(!ii,2)-R{!ss}(!ii,1) +1 ) then
							checkall_flip(!ii,!ss) = 1
						endif 	
					else
		
					if R{!ss}(!ii,3) = -1 then
						!check=0
						for !rr=R{!ss}(!ii,1) to R{!ss}(!ii,2)
							if {%irfname}{!ss}(!ii,!rr)<0 then
								!check=!check+1
							endif
						next
						
						if !check=(R{!ss}(!ii,2)-R{!ss}(!ii,1)+1) then
							checkall(!ii,!ss) = 1
						endif 
	
						' Check flipped signs
						!check_flip=0
						for !rr=R{!ss}(!ii,1) to R{!ss}(!ii,2)
							if {%irfname}{!ss}(!ii,!rr)>0 then
								!check_flip = !check_flip+1
							endif
						next
	
						if !check_flip=(R{!ss}(!ii,2)-R{!ss}(!ii,1)+1) then
							checkall_flip(!ii,!ss) = 1
						endif 
					endif
					endif
					endif
				endif
			next
	
		next
	
		if @mean(checkall)=1 then 'all restrictions satisfied
			!jj = !jj+1
	
			'Rearrange impulses to store in large matrix of successful draws
			if !aux=1 then 'do FEVD and Hdecomp
	
			call VARFevd(Fcomp,invAirf_draw,sigma_draw)
			call VARHDecomp(vresiduals, invAirf_draw,Fcomp)
			
			
				for !rrr = 1 to !nshocks
					for !ccc = 1 to !nvar
						matrix tempstoreirf = @subextract({%irfname}{!rrr},!ccc,1,!ccc,!nsteps)
						matplace(irf{!rrr}_{!ccc},tempstoreirf,!jj,1)
						matplace(fevd{!ccc}_{!rrr},@subextract(@transpose(fevd{!ccc}),!rrr,1,!rrr,!nsteps),!jj,1)
						matplace(hd{!ccc}_{!rrr},@subextract(@transpose(hshock{!ccc}),!rrr,1,!rrr,!nobs+!nlag),!jj,1)
					next
				next
	
				'delete hshock{!ccc}
				'delete FEVD{!ccc}
			
				'Display number of loops
					if !jj=10*!ww then
						logmsg Loop: !jj / !kkk draws
						!ww=!ww+1
					endif
	
			else 'Do not do FEVD and HDecomp just impulses
	
				for !rrr = 1 to !nshocks
					for !ccc = 1 to !nvar
						matrix tempstoreirf = @subextract({%irfname}{!rrr},!ccc,1,!ccc,!nsteps)
						matplace(irf{!rrr}_{!ccc},tempstoreirf,!jj,1)
					next
				next
				
				'Display number of loops
					if !jj=10*!ww then
						logmsg Loop: !jj / !kkk draws
						!ww=!ww+1
					endif
			endif
		else
	
		if @mean(checkall_flip)=1 then 'all restrictions satisfied (but with all oppositive signs)
			!jj = !jj+1
	
			'Rearrange impulses to store in large matrix of successful draws
			if !aux=1 then 'do FEVD and Hdecomp
	
			call VARFevd(Fcomp,invAirf_draw,sigma_draw)
			call VARHDecomp(vresiduals, invAirf_draw,Fcomp)
			
				for !rrr = 1 to !nshocks
					{%irfname}{!rrr}=-{%irfname}{!rrr} 'Change the sign of the impulse (it's flipped)
					for !ccc = 1 to !nvar
						matrix tempstoreirf = @subextract({%irfname}{!rrr},!ccc,1,!ccc,!nsteps)
						matplace(irf{!rrr}_{!ccc},tempstoreirf,!jj,1)
						matplace(fevd{!ccc}_{!rrr},@subextract(@transpose(fevd{!ccc}),!rrr,1,!rrr,!nsteps),!jj,1)
						matplace(hd{!ccc}_{!rrr},@subextract(@transpose(hshock{!ccc}),!rrr,1,!rrr,!nobs+!nlag),!jj,1)
					next
				next
	
				'delete hshock{!ccc}
				'delete FEVD{!ccc}
				
				'Display number of loops
					if !jj=10*!ww then
						logmsg Loop: !jj / !kkk draws
						!ww=!ww+1
					endif
			else
	
				for !rrr = 1 to !nshocks
					{%irfname}{!rrr}=-{%irfname}{!rrr} 'Change the sign of the impulse (it's flipped)
					for !ccc = 1 to !nvar
						matrix tempstoreirf = @subextract({%irfname}{!rrr},!ccc,1,!ccc,!nsteps)
						matplace(irf{!rrr}_{!ccc},tempstoreirf,!jj,1)
					next
				next
				
				'Display number of loops
					if !jj=10*!ww then
						logmsg Loop: !jj / !kkk draws
						!ww=!ww+1
					endif
	
			endif
		endif
	endif
	
	'Reset Ft_draw and inv_sigma matrices, otherwise they seem to get cumulated during the loop for some reason
	Ft_hat=Ft_hat_hold
	inv_sigma_hat=inv_sigma_hat_hold
	delete Fcomp_draw
	delete Ft_draw
	delete sigma_draw
	
	wend 'Finished sign restriction procedure
	
	'Create Summaries of impulses, FEVD etc (median, lower and upper percentiles)
	'================================================================
	
	'use @columnextract to pull out each column 1x1 and take the median then compile into one median and for pctile ranges
	for !shock=1 to !nshocks
		for !response = 1 to !nvar
			for !r = 1 to (!nobs+!nlag)
				if !r<!nsteps+1 then
					'Impulses
					mIRF{!shock}_{!response}(1,!r) = @median(@columnextract(irf{!shock}_{!response},!r))
					mIRFLow{!shock}_{!response}(1,!r) = @quantile(@columnextract(irf{!shock}_{!response},!r),!pctilel)
					mIRFUpp{!shock}_{!response}(1,!r) = @quantile(@columnextract(irf{!shock}_{!response},!r),!pctileu)
	
					if !aux=1 then
						'FEVD
						mFEVD{!response}_{!shock}(1,!r) = @median(@columnextract(FEVD{!response}_{!shock},!r))
						mFEVDLow{!response}_{!shock}(1,!r) = @quantile(@columnextract(FEVD{!response}_{!shock},!r),!pctilel)
						mFEVDUpp{!response}_{!shock}(1,!r) = @quantile(@columnextract(FEVD{!response}_{!shock},!r),!pctileu)
						'HDecomp
						mHD{!response}_{!shock}(1,!r) = @median(@columnextract(Hd{!response}_{!shock},!r))
						mHDLow{!response}_{!shock}(1,!r) = @quantile(@columnextract(Hd{!response}_{!shock},!r),!pctilel)
						mHDUpp{!response}_{!shock}(1,!r) = @quantile(@columnextract(Hd{!response}_{!shock},!r),!pctileu)
					endif
				else
					if !aux=1 then
						'HDecomp
						mHD{!response}_{!shock}(1,!r) = @median(@columnextract(Hd{!response}_{!shock},!r))
						mHDLow{!response}_{!shock}(1,!r) = @quantile(@columnextract(Hd{!response}_{!shock},!r),!pctilel)
						mHDUpp{!response}_{!shock}(1,!r) = @quantile(@columnextract(Hd{!response}_{!shock},!r),!pctileu)
					endif
				endif										
			next
		next
	next
	
endsub


