subroutine VARprocs
	'VAR Procedures for Sign Restriction Code
	'Program to collect and construct useful matrices and strings from the VAR and System Objects
	
	'VAR objects
	'============================================
	!coefs = {%varname}.@eqncoef(1) 'number of estimated coefficients per equation
	!nvarx = (!coefs-(!nvar*!nlag)) 'number of exogenous vars
	!nvarXeq = !coefs-!nvarx 'number of endogenous coefficients
	!nobs={%varname}.@eqregobs(1) 'number of observations in the VAR
	!wishdof = !nobs - !coefs 'degrees of freedom for wishart random number
	string varsmpl={%varname}.@smpl 'var sample period
	{%varname}.makeresids'generate residuals
	group resids resid* 'group the residuals
	resids.drop resid 'drop eviews default resid series
	stom(resids,vresiduals) 'convert the VAR residuals to matrix object
	delete resid* 'delete residuals group
	
	'Make VAR objects
	'============================================
	matrix(!nobs,1) mconst=1 'create vector of ones
	'save the endogous variables used in the VAR
	{%varname}.makeendog yvars
	for !abc=1 to !nvar
		string ser{!abc}=yvars.@seriesname(!abc)
	next
	stom(yvars,ymat) 'convert the endog variables into a matrix
	matrix Ctx = {%varname}.@coefmat 'matrix of coefficients from the estimated VAR
	{%varname}.makemodel(varmod) 'convert VAR object to a model object
	
	' Assemble VAR data into matrix for impulses/fevd/hdecomp
	'================================================
	'Create Matrix of data in the VAR system
	matrix(!nobs,(!nvar*!nlag+!nvarx)) vardata
	
	'Collect useful VAR data procs
	string s_exvars=varmod.@exoglist 'save list of exogenous variables from VAR
	group exvars {s_exvars} 'group exog variables into group
	matrix(!nobs,!nvarXeq) exogvars=na 'create exogenous variables matrix
	sym sigma = {%varname}.@residcov 'save residual cov/var matrix from VAR
	matrix invAirf = @cholesky(sigma) 'Cholesky of the residual cov/var matrix
	matrix(!nsteps,!nvar) SE = 0 'For use in the Hdecomp proc
	close {%varname} 'close the VAR object
	
	'Place appropriate exog variables into data matrix
	if !setexog=1 then
		stom(const,exmat)
	else
        if !setexog<>0 then
		stom(exvars,exmat)
	endif
        endif
	
	'Place exogenous data into VAR
	if !setexog=0 then 'no constant
		'Nothing
	else
	if !setexog=1 then 'constnat
		matplace(vardata,mconst,1,1)
	else
	if !setexog=2 or !setexog=3 then 'constant and trends
		matplace(vardata,mconst,1,1)
		matplace(vardata,@subextract(exmat,!nlag+1,1),1,2)
	endif
	endif
	endif
	
	!firstdata=!nlag+1
	!start=0
	
	'Loop to place endog variables into the matrix ordered by lag
	for !ii=1 to !nvar
		!start=!start+1
		!cc=!ii
		
		for !tt=1 to !nlag	
			if !tt>1 then
				!cc=!cc+!nvar
			endif
			matplace(vardata,@subextract(ymat,!firstdata-!tt,!ii,!nobs+!nlag-!tt,!ii),1,!nvarx+!cc)
			next
	next
	
	'Estimate VAR in System Object (provides a nice way of getting vector of coefficients in nicer ordering)
	'===============================================================================
	{%varname}.makesystem(bylag, n=syschol) 'make system, oredered BY LAG
	c=0
	syschol.ls 'estimate by OLS
	syschol.makeresids 'save residuals
	group resids resid* 'group the residuals
	resids.drop resid 'drop eviews default resid series
	stom(resids,sresiduals) 'convert residual series to matrix of residuals
	delete resid* 'delete residuals
	matrix Csx = syschol.@coefs 'create a matrix of estimated coefficients
	
	'Setup Coefficient Matrix in different ordering (exog first, ordered by variable, then lag)
	'==================================================================
	matrix(!nvar*!nlag,!nvar) Fs
	
	!a=0
	for !cc = 1 to !nvar
		for !rr = 1 to (!nvar*!nlag)
				Fs(!rr,!cc) = Csx(!rr+!a) 
		next
		!a = !a + (!nvar*!nlag + !nvarx)
	next 
	
	matrix(!coefs,!nvar) Ft 'matrix of coefficients ordered with exogenous variables first
	
	if !setexog=0 then
		matplace(Ft,Fs,1+!nvarx,1)
	else
		matplace(Ft,@subextract(Ctx,!coefs-!nvarx+1,1,!coefs,!nvar))
		matplace(Ft,Fs,1+!nvarx,1)
	endif
	
	'Create Companion Matrix for Impulse Responses
	'===================================================================
	matrix Fst=@transpose(Fs) 'transpose coefficient matrix
	matrix(!nvar*!nlag,!nvar*!nlag) Fcomp = 0 'create the companion matrix
	matplace(Fcomp,Fst,1,1) 'place some coefficients in correct spot in comp. matrix
	
	!r=!nvar
	!c=0
	!zzz=0
	
	'Add 1s to relevant cells of the comp. matrix
	while !zzz < !nvar*!nlag-!nvar
		!zzz=!zzz+1
		!r=!r+1
		!c=!c+1
		Fcomp(!r,!c)=1
	wend
	
	'Store Original Coefficient and Covariance Matrix for VARdraw proc
	matrix Ft_hat = Ft
	sym inv_sigma_hat = @inverse(sigma)
	matrix Ft_hat_hold = Ft_hat
	sym inv_sigma_hat_hold = inv_sigma_hat

	'Misc Objects
	'===================================================================	
	string lowerb = @str(!pctilel*100)+"th "+"percentile"
	string upperb = @str(!pctileu*100)+"th "+"percentile"

endsub


